@@ -21,23 +21,23 @@ import sys
 import textwrap
 import types
 import warnings
+from collections.abc import MutableMapping, Sequence
 from functools import partial, wraps
 from io import StringIO
 from keyword import iskeyword
 from random import _inst as global_random_instance
 from tokenize import COMMENT, detect_encoding, generate_tokens, untokenize
 from types import ModuleType
-from typing import TYPE_CHECKING, Any, Callable, MutableMapping
+from typing import Any, Callable, TypeVar
 from unittest.mock import _patch as PatchType
 from weakref import WeakKeyDictionary
 
 from hypothesis.errors import HypothesisWarning
-from hypothesis.internal.compat import PYPY, is_typed_named_tuple
+from hypothesis.internal.compat import is_typed_named_tuple
 from hypothesis.utils.conventions import not_set
 from hypothesis.vendor.pretty import pretty
 
-if TYPE_CHECKING:
-    from hypothesis.strategies._internal.strategies import T
+T = TypeVar("T")
 
 READTHEDOCS = os.environ.get("READTHEDOCS", None) == "True"
 LAMBDA_SOURCE_CACHE: MutableMapping[Callable, str] = WeakKeyDictionary()
@@ -67,14 +67,7 @@ def _clean_source(src: str) -> bytes:
     # lines - i.e. any decorators, so that adding `@example()`s keeps the same key.
     try:
         funcdef = ast.parse(src).body[0]
-        if sys.version_info[:2] == (3, 8) and PYPY:
-            # We can't get a line number of the (async) def here, so as a best-effort
-            # approximation we'll use str.split instead and hope for the best.
-            tag = "async def " if isinstance(funcdef, ast.AsyncFunctionDef) else "def "
-            if tag in src:
-                src = tag + src.split(tag, maxsplit=1)[1]
-        else:
-            src = "".join(src.splitlines(keepends=True)[funcdef.lineno - 1 :])
+        src = "".join(src.splitlines(keepends=True)[funcdef.lineno - 1 :])
     except Exception:
         pass
     # Remove blank lines and use the tokenize module to strip out comments,
@@ -89,7 +82,7 @@ def _clean_source(src: str) -> bytes:
     return "\n".join(x.rstrip() for x in src.splitlines() if x.rstrip()).encode()
 
 
-def function_digest(function):
+def function_digest(function: Any) -> bytes:
     """Returns a string that is stable across multiple invocations across
     multiple processes and is prone to changing significantly in response to
     minor changes to the function.
@@ -166,16 +159,6 @@ def get_signature(
                     parameters=[v for k, v in sig.parameters.items() if k != "self"]
                 )
         return sig
-    if sys.version_info[:2] <= (3, 8) and inspect.isclass(target):
-        # Workaround for subclasses of typing.Generic on Python <= 3.8
-        from hypothesis.strategies._internal.types import is_generic_type
-
-        if is_generic_type(target):
-            sig = inspect.signature(target.__init__)
-            check_signature(sig)
-            return sig.replace(
-                parameters=[v for k, v in sig.parameters.items() if k != "self"]
-            )
     # eval_str is only supported by Python 3.10 and newer
     if sys.version_info[:2] >= (3, 10):
         sig = inspect.signature(
@@ -231,7 +214,9 @@ def convert_keyword_arguments(function, args, kwargs):
     return bound.args, bound.kwargs
 
 
-def convert_positional_arguments(function, args, kwargs):
+def convert_positional_arguments(
+    function: Any, args: Sequence[object], kwargs: dict[str, object]
+) -> tuple[tuple[object, ...], dict[str, object]]:
     """Return a tuple (new_args, new_kwargs) where all possible arguments have
     been moved to kwargs.
 
@@ -257,7 +242,7 @@ def ast_arguments_matches_signature(args, sig):
     assert isinstance(args, ast.arguments)
     assert isinstance(sig, inspect.Signature)
     expected = []
-    for node in getattr(args, "posonlyargs", ()):  # New in Python 3.8
+    for node in args.posonlyargs:
         expected.append((node.arg, inspect.Parameter.POSITIONAL_ONLY))
     for node in args.args:
         expected.append((node.arg, inspect.Parameter.POSITIONAL_OR_KEYWORD))
@@ -452,12 +437,12 @@ def extract_lambda_source(f):
     return source
 
 
-def get_pretty_function_description(f):
+def get_pretty_function_description(f: object) -> str:
     if isinstance(f, partial):
         return pretty(f)
     if not hasattr(f, "__name__"):
         return repr(f)
-    name = f.__name__
+    name = f.__name__  # type: ignore # validated by hasattr above
     if name == "<lambda>":
         return extract_lambda_source(f)
     elif isinstance(f, (types.MethodType, types.BuiltinMethodType)):
@@ -484,7 +469,9 @@ def nicerepr(v):
         return re.sub(r"(\[)~([A-Z][a-z]*\])", r"\g<1>\g<2>", pretty(v))
 
 
-def repr_call(f, args, kwargs, *, reorder=True):
+def repr_call(
+    f: Any, args: Sequence[object], kwargs: dict[str, object], *, reorder: bool = True
+) -> str:
     # Note: for multi-line pretty-printing, see RepresentationPrinter.repr_call()
     if reorder:
         args, kwargs = convert_positional_arguments(f, args, kwargs)
@@ -664,7 +651,7 @@ def impersonate(target):
     return accept
 
 
-def proxies(target: "T") -> Callable[[Callable], "T"]:
+def proxies(target: T) -> Callable[[Callable], T]:
     replace_sig = define_function_signature(
         target.__name__.replace("<lambda>", "_lambda_"),  # type: ignore
         target.__doc__,
@@ -677,6 +664,6 @@ def proxies(target: "T") -> Callable[[Callable], "T"]:
     return accept
 
 
-def is_identity_function(f):
+def is_identity_function(f: object) -> bool:
     # TODO: pattern-match the AST to handle `def ...` identity functions too
     return bool(re.fullmatch(r"lambda (\w+): \1", get_pretty_function_description(f)))
